//
//  MyFlowLayout.m
//  卡片效果
//
//  Created by 吴灶洲 on 2017/1/18.
//  Copyright © 2017年 吴灶洲. All rights reserved.
//

#import "MyFlowLayout.h"

@implementation MyFlowLayout

- (instancetype)initWithItemW:(CGFloat)itemW itemH:(CGFloat)itemH {
    self = [super init];
    if (self) {
        _itemH = itemH;
        _itemW = itemW;
    }
    return self;
}


/** 一些初始化工作，最好在这里实现 */
- (void)prepareLayout {
    [super prepareLayout];
    
    //初始化
    self.itemSize = CGSizeMake(_itemW, _itemH);
    self.scrollDirection = UICollectionViewScrollDirectionHorizontal;
    self.minimumLineSpacing = 100;
    CGFloat inset = (self.collectionView.frame.size.width - _itemW) / 2;
    self.sectionInset = UIEdgeInsetsMake(0, inset, 0, inset);
}



/**
 控制最后srollview的最后去哪里
 
 @param proposedContentOffset 原本Scrollview停止滚动那一刻的位置
 @param velocity 滚动速度
 @return 用来设置collectionView停止滚动那一刻的位置
 */
- (CGPoint)targetContentOffsetForProposedContentOffset:(CGPoint)proposedContentOffset withScrollingVelocity:(CGPoint)velocity {
    //计算scrollview最后停留的范围
    CGRect lastRect ;
    lastRect.origin = proposedContentOffset;
    lastRect.size = self.collectionView.frame.size;
    
    //取出这个范围内的所有属性
    NSArray *array = [self layoutAttributesForElementsInRect:lastRect];
    
    //计算屏幕最中间的x
    CGFloat centerX = proposedContentOffset.x + self.collectionView.frame.size.width / 2 ;
    
    //遍历所有的属性
    CGFloat adjustOffsetX = MAXFLOAT;
    for (UICollectionViewLayoutAttributes *attrs in array) {
        if(ABS(attrs.center.x - centerX) < ABS(adjustOffsetX)){//取出最小值
            adjustOffsetX = attrs.center.x - centerX;
        }
    }
    
    return CGPointMake(proposedContentOffset.x + adjustOffsetX, proposedContentOffset.y);
}


/** 返回yes，只要显示的边界发生改变，就需要重新布局：(会自动调用layoutAttributesForElementsInRect方法，获得所有cell的布局属性) */
- (BOOL)shouldInvalidateLayoutForBoundsChange:(CGRect)newBounds {
    return YES;
}

- (NSArray<UICollectionViewLayoutAttributes *> *)layoutAttributesForElementsInRect:(CGRect)rect {
    //计算可见的矩形框
    CGRect visiableRect;
    visiableRect.size = self.collectionView.frame.size;
    visiableRect.origin = self.collectionView.contentOffset;
    
    //    UICollectionViewLayoutAttributes
    //取出默认cell的UICollectionViewLayoutAttributes
    NSArray *array = [super layoutAttributesForElementsInRect:rect];
    
    //计算屏幕最中间的x
    CGFloat centerX = self.collectionView.contentOffset.x + self.collectionView.frame.size.width / 2 ;
    
    //遍历所有的布局属性
    for (UICollectionViewLayoutAttributes *attrs in array) {
        //不是可见范围的 就返回，不再屏幕就直接跳过
        if (!CGRectIntersectsRect(visiableRect, attrs.frame)) continue;
        
        //每一个item的中心x值
        CGFloat itemCenterx = attrs.center.x;
        //差距越小，缩放比例越大
        //根据与屏幕最中间的距离计算缩放比例
        CGFloat scale = 0.5 + (1 - ABS(itemCenterx - centerX) / self.collectionView.frame.size.width * 0.6)*0.8;//比例值很随意，适合就好

        
        //用这个，缩放不会改变frame大小，所以判断可见范围就无效，item即将离开可见范围的时候，突然消失不见
        attrs.transform3D = CATransform3DMakeScale(scale, scale, 1.0);
    }
    
    return array;
}

@end
